%option noyywrap

%{

/* Copyright 2001 TAKESHIMA Hidenori <hidenori@a2.ctktv.ne.jp> */

/*
  input: C++ style definitions of interface(s).
  output: wine style ICOM definitions.

  interface
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "y.tab.h"

#define	tVIRTUAL	tSPACE
#define	tFAR		tSPACE
#define	tPURE		tSPACE
#define	tSTDCALL	tSPACE
#define	tARG_IN		tSPACE
#define	tARG_OUT	tSPACE

static int lines = 1;

void yyerror( const char* pmsg )
{
	fprintf( stderr, "line %d: %s", lines, pmsg );
	exit(1);
}

void* xmalloc( size_t siz )
{
	void*	p;

	p = malloc( siz );
	if ( p == NULL )
		yyerror( "out of memory.\n" );
	return p;
}

char* xstrdup( const char* pstr )
{
	char*	pdup;
	pdup = (char*)xmalloc( strlen(pstr)+1 );
	memcpy(pdup,pstr,strlen(pstr)+1);
	return pdup;
}

static int handle_cident( const char* ptext )
{
	if ( !strcmp( ptext, "export" ) )
		return tEXPORT;
	if ( !strcmp( ptext, "interface" ) )
		return tINTERFACE;
	if ( !strcmp( ptext, "class" ) )
		return tINTERFACE;
	if ( !strcmp( ptext, "struct" ) )
		return tINTERFACE;
	if ( !strcmp( ptext, "public" ) )
		return tPUBLIC;
	if ( !strcmp( ptext, "virtual" ) )
		return tVIRTUAL;
	if ( !strcmp( ptext, "__stdcall" ) )
		return tSTDCALL;
	if ( !strcmp( ptext, "STDMETHODCALLTYPE" ) )
		return tSTDCALL;
	if ( !strcmp( ptext, "FAR" ) )
		return tFAR;

	if ( !strcmp( ptext, "const" ) )
		return tCONST;
	if ( !strcmp( ptext, "enum" ) )
		return tENUM;

	yylval.str = xstrdup(ptext);
	return tIDENT;
}

%}

ws	[ \f\t\r]
cident	[a-zA-Z_][0-9a-zA-Z_]*

%%

{cident}	{ return handle_cident(yytext); }
{ws}+	return tSPACE;
\/\/[^\n]*	return tSPACE;
\*	return tPTR;
\{	return tIFBEGIN;
\}	return tIFEND;
\n	{ lines++; return tSPACE; }
\(	return tARGBEGIN;
\)	return tARGEND;
\,	return tARGSEP;
\:	return tBEGINBASE;
\;	return tEOL;
\={ws}+0	return tPURE;
\[in\]	return tARG_IN;
\[out\]	return tARG_OUT;

%%
